@@ -28,7 +28,6 @@
 #include "goafacebookprovider.h"
 #include "goaimapsmtpprovider.h"
 #include "goaowncloudprovider.h"
-#include "goayahooprovider.h"
 #include "goaflickrprovider.h"
 #include "goafoursquareprovider.h"
 #include "goawindowsliveprovider.h"
@@ -797,9 +796,6 @@ static struct
 #ifdef GOA_MEDIA_SERVER_ENABLED
   { GOA_MEDIA_SERVER_NAME, goa_media_server_provider_get_type },
 #endif
-#ifdef GOA_YAHOO_ENABLED
-  { GOA_YAHOO_NAME, goa_yahoo_provider_get_type },
-#endif
 #ifdef GOA_TELEPATHY_ENABLED
   { GOA_TELEPATHY_NAME, goa_telepathy_factory_get_type },
 #endif
@@ -1232,6 +1228,7 @@ goa_util_lookup_keyfile_string (GoaObject    *object,
     }
 
  out:
+  g_key_file_free (key_file);
   g_free (group);
   g_free (path);
   return ret;
@@ -1284,6 +1281,7 @@ goa_util_lookup_keyfile_boolean (GoaObject    *object,
     }
 
  out:
+  g_key_file_free (key_file);
   g_free (group);
   g_free (path);
   return ret;
