@@ -1,6 +1,6 @@
 /* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
- * Copyright (C) 2012, 2013, 2015 Red Hat, Inc.
+ * Copyright (C) 2012, 2013 Red Hat, Inc.
  * Copyright (C) 2013 Intel Corporation
  *
  * This library is free software; you can redistribute it and/or
@@ -436,7 +436,7 @@ add_account (GoaProvider  *provider,
   wait_for_account_settings_ready (settings, data.loop);
 
   account_widget = tpaw_account_widget_new_for_protocol (settings,
-      dialog, TRUE);
+      dialog, FALSE);
   gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET (account_widget), FALSE, FALSE, 0);
   gtk_widget_show (GTK_WIDGET (account_widget));
   g_signal_connect (account_widget, "account-created",
@@ -852,7 +852,6 @@ edit_parameters_clicked_cb (GtkButton *button,
 
   if (!edit_connection_parameters (data->object, data->parent, &error))
     maybe_show_error (data->parent, error, _("Cannot save the connection parameters"));
-  g_clear_error (&error);
 }
 
 static void
@@ -865,7 +864,6 @@ edit_personal_details_clicked_cb (GtkButton *button,
   if (!edit_personal_details (data->object, data->parent, &error))
     maybe_show_error (data->parent, error,
         _("Cannot save your personal information on the server"));
-  g_clear_error (&error);
 }
 
 static void
